%
%                            Aeroce_Trace_Elements

%
% $Log: gsoaerot.m,v $
% Revision 1.3  2000/04/04 14:57:32  kwoklin
% Move server from euros to aspre.   klee
%
% Revision 1.2  2000/01/12 18:51:14  kwoklin
% Second checkup.  klee
%
% Revision 1.1  1999/07/22 19:28:06  kwoklin
% Add R. Platner's local data files in.   klee
%

% $Id: gsoaerot.m,v 1.3 2000/04/04 14:57:32 kwoklin Exp $
% Ruth Platner  07/06/99

% FUNCTIONS -- REQUIRED
GetFunctionName = 'getgsod';

% FUNCTIONS -- OPTIONAL
CatalogServer = nan;
URL_m_File = 'gsoaerou';

% VARIABLES -- OPTIONAL
Server = 'http://aspre.gso.uri.edu/cgi-bin/dods/nph-ff/dods/aeroce/trace.dat';

% VARIABLES -- REQUIRED
LonRange = [-70.00 -4.00];
LatRange = [10.00 56.00];
TimeRange = [1988.5 1998.0];
DepthRange = [-46.0 -2375.0];
Resolution = nan;

DataName = 'Aerosol - Daily Time Series at Barbados, Bermuda, Ireland and Tererife - URI/AEROCE';
TimeName = 'time';
LongitudeName = 'lon';
LatitudeName = 'lat';
DepthName = 'height';
DepthUnits = 'Meters';
TextField = str2mat('Bermuda','Barbados','Izania','Mace_Head');
OptionalVariables = NaN;

% these names tie in to the browser as recognized common strings
% linking different datasets
% there must be one DODSNAME for each selectable variable
DodsName1 = str2mat('station','lat','lon','time');
SelectableVariables = str2mat('Trace_Elements');
DodsName            = str2mat('Umbrella');
UmbrellaDods = str2mat(str2mat('Al','Br', ...
'Ca','Cl','Cu','I','Mg','Mn','Na'),str2mat('V','Ba','Co', ...
'Cr','Cs','Eu','Fe','Hf','Nd'),str2mat('Ni','Rb','Sb','Sc', ...
'Se','Ta','Tb','Th','Yb'),'Zn','station');
UmbrellaVariables = str2mat(str2mat('Al','Br', ...
'Ca','Cl','Cu','I','Mg','Mn','Na'),str2mat('V','Ba','Co', ...
'Cr','Cs','Eu','Fe','Hf','Nd'),str2mat('Ni','Rb','Sb','Sc', ...
'Se','Ta','Tb','Th','Yb'),'Zn');

% note: DataNull is applied *before* scaling
DataNull = [NaN NaN NaN NaN ...
               -9.99 -9.99 -9.99 -9.99 ...
-9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 ...
-9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 -9.99 ...
-9.99 -9.99 -9.99 -9.99 -9.99];

Acknowledge = sprintf('%s\n\n%s', ...
    [ 'The author of this dataset is R. Arimoto.  ', ...
      'The data were provided by the University of Rhode Island ', ...
      '(http://euros.gso.uri.edu/dods/aeroce/trace.dat)',...
      'and accessed via the Distributed Oceanographic Data System ',...
      '(DODS: http://www.unidata.ucar.edu/packages/dods).'], ...
    [ 'Citations: Arimoto, R., R. A. Duce, B. J. Ray, ',...
      'W. G. Ellis, Jr., J. D. Cullen, and J. T. Merrill, ',...
      'Trace elements in the atmosphere over the North Atlantic,',...
      'Journal of Geophysical Research, 100, 1199-1213, 1995.']);
Data_Use_Policy = [];

Comments1 = sprintf('%s\n','',...
    '                Daily time series of Trace Elements at Barbados',...
'        ',...
'        Parameter         Description                                     Units',...
'        --------------------------------------------------------------------------',...
'        Aerosols          trace elements measured at std. temp and pres   mg/m^3',...
    '    ',...
    '    The sample data archived here were collected as part of the',...
    '    Atmosphere/Ocean Chemistry Experiment (AEROCE), which is a',...
    '    multi-institution program sponsored by the National Science Foundation.',...
    '    Support for the trace element studies was provided under awards NSF ATM',...
    '    87-02563, ATM 90-13062, ATM 94-14262, and ATM 97-28983.',...
    '    ',...
    '    The data entries are for individual, high-volume, bulk aerosol samples',...
    '    collected on Whatman 41 filters.  A computer-driven control system was used',...
    '    to minimize the collection of locally generated contaminants.',...
    '    The samples were analyzed by instrumental neutron activation analysis using',...
    '    the facilities of the Rhode Island Nuclear Science Center in Narragansett, RI. ');
Comments2 = sprintf('%s\n','',...
    '    All concentrations are expressed in micrograms per cubic meter of air',... 
    '    (at standard temperature and pressure).',...
    '    Time is given in DAYS since December 31, 1987.',...
    '    ',...
    '    Data are available for the following sites:',...
    '    Barbados, Ragged Point 13.18N 59.43W',...
    '    Bermuda,  Tudor Hill   32.24N 64.87W',...
    '    Ireland,  Mace Head    53.43N  9.73W',...
    '    Tererife, Izania       28.30N 16.48W',...
    '    ',...
    '    Information on the analytical methods can be found in the following paper',...   
    '    (and references therein):',...
    '    ',...
    '    Arimoto, R., R. A. Duce, B. J. Ray, W. G. Ellis, Jr., J. D. Cullen, and J.',...
    '    T. Merrill, Trace elements in the atmosphere over the North Atlantic,',...
    '    Journal of Geophysical Research, 100, 1199-1213, 1995.',...
    '    ');
Comments3 = sprintf('%s\n',...
    '    ',...
    '    Time_Period_of_Content = ', ...
    '         Beginning_Date: Jul. 29, 1988 (1988.57)', ...
    '         Ending_Date:    Dec. 31, 1997 (1997.99)', ...
    '    Spatial_Domain = ', ... 
    '         Bounding Coordinates: ', ...
    '              West Bounding Coordinate:   9.73 W ', ...
    '              East Bounding Coordinate:  64.87 W ', ...
    '              South Bounding Coordinate: 13.18 N ', ...
    '              North Bounding Coordinate: 53.43 N ', ...
    '   ', ...
    '    Point_of_Contact (Data Provision) = ', ...
    '         Address - si: ', ...
    '              Dr. Richard Arimoto',...
    '              Carlsbad Environmental Monitoring & Research Center',...
    '              1400 University Drive',...
    '              Carlsbad, NM  88220  ',...
    '              USA',...
    '              Electronic Mail Address:INTERNET > arimoto@cemrc.org',...
    '         Address - dods: ',...
    '              Electronic Mail Address:INTERNET > support@unidata.ucar.edu ');

Comments = [Comments1 Comments2 Comments3];
